@@ -15,6 +15,8 @@
 from typing import Callable, Optional, Sequence, Union
 import warnings
 
+from functools import partial
+
 import jax
 import jax.numpy as jnp
 import jax.experimental.host_callback as hcb
@@ -69,7 +71,7 @@ class TDVP(AbstractVariationalDriver):
             integrator: Configuration of the algorithm used for solving the ODE.
             t0: Initial time at the start of the time evolution.
             propagation_type: Determines the equation of motion: "real"  for the
-                real-time Schödinger equation (SE), "imag" for the imaginary-time SE.
+                real-time Schrödinger equation (SE), "imag" for the imaginary-time SE.
             qgt: The QGT specification.
             linear_solver: The solver for solving the linear system determining the time evolution.
             linear_solver_restart: If False (default), the last solution of the linear system
@@ -224,7 +226,7 @@ class TDVP(AbstractVariationalDriver):
 
         Args:
             T: Length of the integration interval.
-            tstops: A sequence of stopping times, each within the intervall :code:`[self.t0, self.t0 + T]`,
+            tstops: A sequence of stopping times, each within the interval :code:`[self.t0, self.t0 + T]`,
                 at which this method will stop and yield. By default, a stop is performed
                 after each time step (at potentially varying step size if an adaptive
                 integrator is used).
@@ -314,13 +316,13 @@ class TDVP(AbstractVariationalDriver):
             out: A logger object, or an iterable of loggers, to be used to store simulation log and data.
                 If this argument is a string, it will be used as output prefix for the standard JSON logger.
             obs: An iterable containing the observables that should be computed.
-            tstops: A sequence of stopping times, each within the intervall :code:`[self.t0, self.t0 + T]`,
-                at which the driver will stop and perform estimation of observables, logging, and excecute
+            tstops: A sequence of stopping times, each within the interval :code:`[self.t0, self.t0 + T]`,
+                at which the driver will stop and perform estimation of observables, logging, and execute
                 the callback function. By default, a stop is performed after each time step (at potentially
                 varying step size if an adaptive integrator is used).
             show_progress: If true displays a progress bar (default=True)
             callback: Callable or list of callable callback functions to be executed at each
-                stoping time.
+                stopping time.
         """
         if obs is None:
             obs = {}
@@ -362,6 +364,12 @@ class TDVP(AbstractVariationalDriver):
 
                 pbar.n = np.asarray(self._integrator.t)
                 self._postfix["n"] = self.step_count
+                self._postfix.update(
+                    {
+                        self._loss_name: str(self._loss_stats),
+                    }
+                )
+
                 pbar.set_postfix(self._postfix)
                 pbar.refresh()
 
@@ -501,12 +509,17 @@ def odefun(state: MCState, driver: TDVP, t, w, *, stage=0):  # noqa: F811
     state.parameters = w
     state.reset()
 
-    driver._loss_stats, driver._loss_grad = state.expect_and_grad(
-        driver.generator(t),
-        use_covariance=True,
+    op_t = driver.generator(t)
+
+    driver._loss_stats, driver._loss_forces = state.expect_and_forces(
+        op_t,
     )
-    driver._loss_grad = jax.tree_map(
-        lambda x: driver._loss_grad_factor * x, driver._loss_grad
+    driver._loss_grad = _map_parameters(
+        driver._loss_forces,
+        state.parameters,
+        driver._loss_grad_factor,
+        driver.propagation_type,
+        type(state),
     )
 
     qgt = driver.qgt(driver.state)
@@ -517,7 +530,7 @@ def odefun(state: MCState, driver: TDVP, t, w, *, stage=0):  # noqa: F811
     driver._dw, _ = qgt.solve(driver.linear_solver, driver._loss_grad, x0=initial_dw)
 
     # If parameters are real, then take only real part of the gradient (if it's complex)
-    driver._dw = jax.tree_multimap(
+    driver._dw = jax.tree_map(
         lambda x, target: (x if jnp.iscomplexobj(target) else x.real),
         driver._dw,
         state.parameters,
@@ -526,6 +539,24 @@ def odefun(state: MCState, driver: TDVP, t, w, *, stage=0):  # noqa: F811
     return driver._dw
 
 
+@partial(jax.jit, static_argnums=(3, 4))
+def _map_parameters(forces, parameters, loss_grad_factor, propagation_type, state_T):
+
+    forces = jax.tree_map(
+        lambda x, target: loss_grad_factor * x,
+        forces,
+        parameters,
+    )
+
+    forces = jax.tree_map(
+        lambda x, target: (x if jnp.iscomplexobj(target) else x.real),
+        forces,
+        parameters,
+    )
+
+    return forces
+
+
 def odefun_host_callback(state, driver, *args, **kwargs):
     """
     Calls odefun through a host callback in order to make the rest of the
