@@ -179,7 +179,7 @@ class Lattice(Graph):
             point_group: Default `PointGroup` object for constructing space groups
             max_neighbor_order: For :code:`max_neighbor_order == k`, edges between up
                 to :math:`k`-nearest neighbor sites (measured by their Euclidean distance)
-                are included in the graph. The edges can be distiguished by their color,
+                are included in the graph. The edges can be distinguished by their color,
                 which is set to :math:`k - 1` (so nearest-neighbor edges have color 0).
                 By default, nearest neighbours (:code:`max_neighbor_order=1`) are autogenerated
                 unless :code:`custom_edges` is passed.
@@ -314,7 +314,7 @@ class Lattice(Graph):
         basis_vectors = _np.asarray(basis_vectors)
         if basis_vectors.ndim != 2:
             raise ValueError(
-                "'basis_vectors' must have ndim==2 (as array of primtive vectors)"
+                "'basis_vectors' must have ndim==2 (as array of primitive vectors)"
             )
         if basis_vectors.shape[0] != basis_vectors.shape[1]:
             raise ValueError("The number of primitive vectors must match their length")
@@ -377,7 +377,7 @@ class Lattice(Graph):
         ):
             raise ValueError(
                 "pbc must be either a boolean or a sequence of booleans with length"
-                "equal to  the lattice dimenion"
+                "equal to  the lattice dimension"
             )
         else:
             return _np.asarray(pbc, dtype=bool)
@@ -525,7 +525,7 @@ class Lattice(Graph):
         is_valid = _np.logical_or(self.pbc, result == 0)
         if not _np.all(is_valid):
             raise InvalidWaveVectorError(
-                "Some wave vectors are inconisistent with open boundary conditions"
+                "Some wave vectors are inconsistent with open boundary conditions"
             )
 
         return result
@@ -542,7 +542,7 @@ class Lattice(Graph):
         Arguments:
             point_group: a `PointGroup` object describing the point-group
                          symmetries of `self`. Optional, if not supplied, the
-                         `PointGroup` object provded at construction is used.
+                         `PointGroup` object provided at construction is used.
 
         Returns:
             A `SpaceGroupBuilder` object that generates `PermutationGroup`s
