@@ -187,9 +187,7 @@ def value_and_grad(
                         )(*args, **kwargs)
 
                     out = out_r + 1j * out_j
-                    grad = jax.tree_multimap(
-                        lambda re, im: re + 1j * im, grad_r, grad_j
-                    )
+                    grad = jax.tree_map(lambda re, im: re + 1j * im, grad_r, grad_j)
 
                     if has_aux:
                         return out, grad, aux
