@@ -22,7 +22,7 @@ from flax import linen as nn
 from jax.nn.initializers import zeros, lecun_normal
 from jax.scipy.special import logsumexp
 
-from netket.utils import HashableArray, warn_deprecation
+from netket.utils import HashableArray, warn_deprecation, deprecate_dtype
 from netket.utils.types import NNInitFunc
 from netket.utils.group import PermutationGroup
 from netket.graph import Graph, Lattice
@@ -44,10 +44,12 @@ def identity(x):
     return x
 
 
+@deprecate_dtype
 class GCNN_FFT(nn.Module):
     r"""Implements a GCNN using a fast fourier transform over the translation group.
+
     The group convolution can be written in terms of translational convolutions with
-    symmetry transformed filters as desribed in ` Cohen et. *al* <http://proceedings.mlr.press/v48/cohenc16.pdf>`_
+    symmetry transformed filters as described in ` Cohen et. *al* <http://proceedings.mlr.press/v48/cohenc16.pdf>`_
     The translational convolutions are then implemented with Fast Fourier Transforms.
     """
 
@@ -68,7 +70,7 @@ class GCNN_FFT(nn.Module):
     all layers will have the same number of features."""
     characters: HashableArray
     """Array specifying the characters of the desired symmetry representation"""
-    dtype: Any = float
+    param_dtype: Any = float
     """The dtype of the weights."""
     activation: Any = reim_selu
     """The nonlinear activation function between hidden layers."""
@@ -96,7 +98,7 @@ class GCNN_FFT(nn.Module):
             space_group=self.symmetries,
             shape=self.shape,
             features=self.features[0],
-            dtype=self.dtype,
+            param_dtype=self.param_dtype,
             use_bias=self.use_bias,
             kernel_init=self.kernel_init,
             bias_init=self.bias_init,
@@ -109,7 +111,7 @@ class GCNN_FFT(nn.Module):
                 shape=self.shape,
                 features=self.features[layer + 1],
                 use_bias=self.use_bias,
-                dtype=self.dtype,
+                param_dtype=self.param_dtype,
                 precision=self.precision,
                 kernel_init=self.kernel_init,
                 bias_init=self.bias_init,
@@ -140,13 +142,14 @@ class GCNN_FFT(nn.Module):
             return x
 
 
+@deprecate_dtype
 class GCNN_Irrep(nn.Module):
     r"""Implements a GCNN by projecting onto irreducible
     representations of the group. The projection onto
     the group is implemented with matrix multiplication
 
     Layers act on a feature maps of shape [batch_size, in_features, n_symm] and
-    eeturns a feature map of shape [batch_size, features, n_symm].
+    returns a feature map of shape [batch_size, features, n_symm].
     The input and the output are related by
 
     .. math ::
@@ -172,7 +175,7 @@ class GCNN_Irrep(nn.Module):
     Numpy/Jax arrays must be wrapped into an :class:`netket.utils.HashableArray`.
     """
     irreps: Tuple[HashableArray]
-    """List of irreducible represenation matrices"""
+    """List of irreducible representation matrices"""
     layers: int
     """Number of layers (not including sum layer over output)."""
     features: Tuple
@@ -180,7 +183,7 @@ class GCNN_Irrep(nn.Module):
     all layers will have the same number of features."""
     characters: HashableArray
     """Array specifying the characters of the desired symmetry representation"""
-    dtype: Any = np.float64
+    param_dtype: Any = np.float64
     """The dtype of the weights."""
     activation: Any = reim_selu
     """The nonlinear activation function between hidden layers."""
@@ -207,7 +210,7 @@ class GCNN_Irrep(nn.Module):
         self.dense_symm = DenseSymmMatrix(
             symmetries=self.symmetries,
             features=self.features[0],
-            dtype=self.dtype,
+            param_dtype=self.param_dtype,
             use_bias=self.use_bias,
             kernel_init=self.kernel_init,
             bias_init=self.bias_init,
@@ -219,7 +222,7 @@ class GCNN_Irrep(nn.Module):
                 irreps=self.irreps,
                 features=self.features[layer + 1],
                 use_bias=self.use_bias,
-                dtype=self.dtype,
+                param_dtype=self.param_dtype,
                 precision=self.precision,
                 kernel_init=self.kernel_init,
                 bias_init=self.bias_init,
@@ -250,10 +253,11 @@ class GCNN_Irrep(nn.Module):
             return x
 
 
+@deprecate_dtype
 class GCNN_Parity_FFT(nn.Module):
     r"""Implements a GCNN using a fast fourier transform over the translation group.
     The group convolution can be written in terms of translational convolutions with
-    symmetry transformed filters as desribed in ` Cohen et. *al* <http://proceedings.mlr.press/v48/cohenc16.pdf>`_
+    symmetry transformed filters as described in ` Cohen et. *al* <http://proceedings.mlr.press/v48/cohenc16.pdf>`_
     The translational convolutions are then implemented with Fast Fourier Transforms.
     This model adds parity symmetry under the transformation x->-x
     """
@@ -277,7 +281,7 @@ class GCNN_Parity_FFT(nn.Module):
     """Array specifying the characters of the desired symmetry representation"""
     parity: int
     """Integer specifying the eigenvalue with respect to parity"""
-    dtype: Any = np.float64
+    param_dtype: Any = np.float64
     """The dtype of the weights."""
     activation: Any = reim_selu
     """The nonlinear activation function between hidden layers."""
@@ -302,7 +306,7 @@ class GCNN_Parity_FFT(nn.Module):
     `characters` are negative."""
 
     def setup(self):
-        # TODO: evenutally remove this warning
+        # TODO: eventually remove this warning
         # supports a deprecated attribute
         if self.extra_bias:
             warn_deprecation(
@@ -319,7 +323,7 @@ class GCNN_Parity_FFT(nn.Module):
             space_group=self.symmetries,
             shape=self.shape,
             features=self.features[0],
-            dtype=self.dtype,
+            param_dtype=self.param_dtype,
             use_bias=self.use_bias,
             kernel_init=self.kernel_init,
             bias_init=self.bias_init,
@@ -332,7 +336,7 @@ class GCNN_Parity_FFT(nn.Module):
                 shape=self.shape,
                 features=self.features[layer + 1],
                 use_bias=self.use_bias,
-                dtype=self.dtype,
+                param_dtype=self.param_dtype,
                 precision=self.precision,
                 kernel_init=self.kernel_init,
                 bias_init=self.bias_init,
@@ -347,7 +351,7 @@ class GCNN_Parity_FFT(nn.Module):
                 features=self.features[layer + 1],
                 # this would bias the same outputs as self.equivariant
                 use_bias=self.extra_bias and self.use_bias,
-                dtype=self.dtype,
+                param_dtype=self.param_dtype,
                 precision=self.precision,
                 kernel_init=self.kernel_init,
                 bias_init=self.bias_init,
@@ -407,13 +411,14 @@ class GCNN_Parity_FFT(nn.Module):
             return x
 
 
+@deprecate_dtype
 class GCNN_Parity_Irrep(nn.Module):
     r"""Implements a GCNN by projecting onto irreducible
     representations of the group. The projection onto
     the group is implemented with matrix multiplication
 
     Layers act on a feature maps of shape [batch_size, in_features, n_symm] and
-    eeturns a feature map of shape [batch_size, features, n_symm].
+    returns a feature map of shape [batch_size, features, n_symm].
     The input and the output are related by
 
     .. math ::
@@ -442,7 +447,7 @@ class GCNN_Parity_Irrep(nn.Module):
     Numpy/Jax arrays must be wrapped into an :class:`netket.utils.HashableArray`.
     """
     irreps: Tuple[HashableArray]
-    """List of irreducible represenation matrices"""
+    """List of irreducible representation matrices"""
     layers: int
     """Number of layers (not including sum layer over output)."""
     features: Tuple
@@ -452,7 +457,7 @@ class GCNN_Parity_Irrep(nn.Module):
     """Array specifying the characters of the desired symmetry representation"""
     parity: int
     """Integer specifying the eigenvalue with respect to parity"""
-    dtype: Any = np.float64
+    param_dtype: Any = np.float64
     """The dtype of the weights."""
     activation: Any = reim_selu
     """The nonlinear activation function between hidden layers."""
@@ -477,7 +482,7 @@ class GCNN_Parity_Irrep(nn.Module):
     `characters` are negative."""
 
     def setup(self):
-        # TODO: evenutally remove this warning
+        # TODO: eventually remove this warning
         # supports a deprecated attribute
         if self.extra_bias:
             warn_deprecation(
@@ -493,7 +498,7 @@ class GCNN_Parity_Irrep(nn.Module):
         self.dense_symm = DenseSymmMatrix(
             symmetries=self.symmetries,
             features=self.features[0],
-            dtype=self.dtype,
+            param_dtype=self.param_dtype,
             use_bias=self.use_bias,
             kernel_init=self.kernel_init,
             bias_init=self.bias_init,
@@ -505,7 +510,7 @@ class GCNN_Parity_Irrep(nn.Module):
                 irreps=self.irreps,
                 features=self.features[layer + 1],
                 use_bias=self.use_bias,
-                dtype=self.dtype,
+                param_dtype=self.param_dtype,
                 precision=self.precision,
                 kernel_init=self.kernel_init,
                 bias_init=self.bias_init,
@@ -519,7 +524,7 @@ class GCNN_Parity_Irrep(nn.Module):
                 features=self.features[layer + 1],
                 # this would bias the same outputs as self.equivariant
                 use_bias=self.extra_bias and self.use_bias,
-                dtype=self.dtype,
+                param_dtype=self.param_dtype,
                 precision=self.precision,
                 kernel_init=self.kernel_init,
                 bias_init=self.bias_init,
@@ -579,6 +584,7 @@ class GCNN_Parity_Irrep(nn.Module):
             return x
 
 
+@deprecate_dtype
 def GCNN(
     symmetries=None,
     product_table=None,
@@ -590,7 +596,7 @@ def GCNN(
     features=None,
     characters=None,
     parity=None,
-    dtype=np.float64,
+    param_dtype=np.float64,
     complex_output=True,
     **kwargs,
 ):
@@ -609,17 +615,18 @@ def GCNN(
 
         {\bf f}^{i+1}_h = \Gamma( \sum_h W_{g^{-1} h} {\bf f}^i_h).
 
+
     Args:
         symmetries: A specification of the symmetry group. Can be given by a
-            nk.graph.Graph, a nk.utils.PermuationGroup, or an array [n_symm, n_sites]
-            specifying the permutations corresponding to symmetry transformations
-            of the lattice.
+            :class:`nk.graph.Graph`, a :class:`nk.utils.PermutationGroup`, or an
+            array :code:`[n_symm, n_sites]` specifying the permutations
+            corresponding to symmetry transformations of the lattice.
         product_table: Product table describing the algebra of the symmetry group.
             Only needs to be specified if mode='fft' and symmetries is specified as an array.
         irreps: List of 3D tensors that project onto irreducible representations of the symmetry group.
             Only needs to be specified if mode='irreps' and symmetries is specified as an array.
-        point_group: The point group, from which the space group is built.
-            If symmetries is a graph the default point group is overwritten.
+        point_group: The point group, from which the space group is built. If symmetries is a
+            graph the default point group is overwritten.
         mode: string "fft, irreps, matrix, auto" specifying whether to use a fast
             fourier transform over the translation group, a fourier transform using
             the irreducible representations or by constructing the full kernel matrix.
@@ -627,23 +634,25 @@ def GCNN(
         layers: Number of layers (not including sum layer over output).
         features: Number of features in each layer starting from the input. If a single
             number is given, all layers will have the same number of features.
-        characters: Array specifying the characters of the desired symmetry representation
+        characters: Array specifying the characters of the desired symmetry representation.
         parity: Optional argument with value +/-1 that specifies the eigenvalue
             with respect to parity (only use on two level systems).
-        dtype: The dtype of the weights.
+        param_dtype: The dtype of the weights.
         activation: The nonlinear activation function between hidden layers. Defaults to
-            :ref:`nk.nn.activation.reim_selu`.
+            :func:`nk.nn.activation.reim_selu` .
         output_activation: The nonlinear activation before the output.
         equal_amplitudes: If True forces all basis states to have equal amplitude
-            by setting :math:`\Re(\psi) = 0`.
+            by setting :math:`\Re(\psi) = 0` .
         use_bias: If True uses a bias in all layers.
-        precision: Numerical precision of the computation see `jax.lax.Precision`for details.
+        precision: Numerical precision of the computation see :class:`jax.lax.Precision` for details.
         kernel_init: Initializer for the kernels of all layers. Defaults to
-            `lecun_normal(in_axis=1, out_axis=0)` which guarantees the correct variance of the
+            :code:`lecun_normal(in_axis=1, out_axis=0)` which guarantees the correct variance of the
             output.
         bias_init: Initializer for the biases of all layers.
         complex_output: If True, ensures that the network output is always complex.
             Necessary when network parameters are real but some `characters` are negative.
+
+
     """
 
     if isinstance(symmetries, Lattice) and (
@@ -700,7 +709,7 @@ def GCNN(
     else:
         if (
             not is_complex(characters)
-            and not is_complex_dtype(dtype)
+            and not is_complex_dtype(param_dtype)
             and not complex_output
             and jnp.any(characters < 0)
         ):
@@ -723,7 +732,7 @@ def GCNN(
                 characters=characters,
                 shape=shape,
                 parity=parity,
-                dtype=dtype,
+                param_dtype=param_dtype,
                 complex_output=complex_output,
                 **kwargs,
             )
@@ -735,7 +744,7 @@ def GCNN(
                 features=features,
                 characters=characters,
                 shape=shape,
-                dtype=dtype,
+                param_dtype=param_dtype,
                 complex_output=complex_output,
                 **kwargs,
             )
@@ -753,7 +762,7 @@ def GCNN(
                 features=features,
                 characters=characters,
                 parity=parity,
-                dtype=dtype,
+                param_dtype=param_dtype,
                 complex_output=complex_output,
                 **kwargs,
             )
@@ -764,7 +773,7 @@ def GCNN(
                 layers=layers,
                 features=features,
                 characters=characters,
-                dtype=dtype,
+                param_dtype=param_dtype,
                 complex_output=complex_output,
                 **kwargs,
             )
