@@ -1,13 +1,17 @@
 import flax.linen as nn
 import jax.numpy as jnp
-from jax.nn.initializers import normal
 
+from flax.linen.dtypes import promote_dtype
+from flax.linen.initializers import normal
+
+from netket.utils import deprecate_dtype
 from netket.utils.types import DType, Array, NNInitFunc
 
 
+@deprecate_dtype
 class Gaussian(nn.Module):
     r"""
-    Multivariate Gaussain function with mean 0 and parametrised covariance matrix
+    Multivariate Gaussian function with mean 0 and parametrised covariance matrix
     :math:`\Sigma_{ij}`.
 
     The wavefunction is given by the formula: :math:`\Psi(x) = \exp(\sum_{ij} x_i \Sigma_{ij} x_j)`.
@@ -15,7 +19,7 @@ class Gaussian(nn.Module):
     non-positive definite matrix A.
     """
 
-    dtype: DType = jnp.float64
+    param_dtype: DType = jnp.float64
     """The dtype of the weights."""
     kernel_init: NNInitFunc = normal(stddev=1.0)
     """Initializer for the weights."""
@@ -24,14 +28,10 @@ class Gaussian(nn.Module):
     def __call__(self, x_in: Array):
         nv = x_in.shape[-1]
 
-        dtype = jnp.promote_types(x_in.dtype, self.dtype)
-        x_in = jnp.asarray(x_in, dtype=dtype)
-
-        kernel = self.param("kernel", self.kernel_init, (nv, nv), self.dtype)
-
+        kernel = self.param("kernel", self.kernel_init, (nv, nv), self.param_dtype)
         kernel = jnp.dot(kernel.T, kernel)
 
-        # print(kernel)
+        kernel, x_in = promote_dtype(kernel, x_in, dtype=None)
         y = -0.5 * jnp.einsum("...i,ij,...j", x_in, kernel, x_in)
 
         return y
