@@ -14,7 +14,7 @@
 
 from typing import Optional
 from functools import partial
-from netket.jax import compose, vmap_chunked
+import math
 
 import jax
 import jax.flatten_util
@@ -24,10 +24,15 @@ import numpy as np
 
 from netket.stats import subtract_mean, sum
 from netket.utils import mpi
-
 from netket.utils.types import Array, Callable, PyTree, Scalar
-
-from netket.jax import tree_cast, tree_conj, tree_axpy, tree_to_real
+from netket.jax import (
+    tree_cast,
+    tree_conj,
+    tree_axpy,
+    tree_to_real,
+    compose,
+    vmap_chunked,
+)
 
 
 # TODO better name and move it somewhere sensible
@@ -114,7 +119,8 @@ def _divide_by_sqrt_n_samp(oks, samples):
     divide Oⱼₖ by √n
     """
     n_samp = samples.shape[0] * mpi.n_nodes  # MPI
-    return jax.tree_map(lambda x: x / np.sqrt(n_samp, dtype=x.dtype), oks)
+    sqrt_n = math.sqrt(n_samp)  # enforce weak type
+    return jax.tree_map(lambda x: x / sqrt_n, oks)
 
 
 def _multiply_by_pdf(oks, pdf):
@@ -227,7 +233,7 @@ def prepare_centered_oks(
         mode: differentiation mode, must be one of 'real', 'complex', 'holomorphic'
         rescale_shift: whether scale-invariant regularisation should be used (default: True)
         pdf: |ψ(x)|^2 if exact optimization is being used else None
-        chunk_size: an int specfying the size of the chunks the gradient should be computed in (default: None)
+        chunk_size: an int specifying the size of the chunks the gradient should be computed in (default: None)
 
     Returns:
         if not rescale_shift:
