@@ -12,7 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from .base import VariationalState, VariationalMixedState, expect, expect_and_grad
+from .base import (
+    VariationalState,
+    VariationalMixedState,
+    expect,
+    expect_and_grad,
+    expect_and_forces,
+)
 
 from .mc import MCState, MCMixedState, get_local_kernel_arguments, get_local_kernel
 from .exact import ExactState
